@@ -1,4 +1,4 @@
-// This file is part of VSTGUI. It is subject to the license terms 
+// This file is part of VSTGUI. It is subject to the license terms
 // in the LICENSE file found in the top-level directory of this
 // distribution and at http://github.com/steinbergmedia/vstgui/LICENSE
 
@@ -13,9 +13,9 @@
 
 namespace VSTGUI {
 #if TARGET_OS_IPHONE
-static const float kCKnobRange = 300.f;
+static const float kCKnobRangeDefault = 300.f;
 #else
-static const float kCKnobRange = 200.f;
+static const float kCKnobRangeDefault = 200.f;
 #endif
 
 static constexpr CViewAttributeID kCKnobMouseStateAttribute = 'knms';
@@ -40,6 +40,7 @@ CKnobBase::CKnobBase (const CRect& size, IControlListener* listener, int32_t tag
 	setStartAngle ((float)(3.f * Constants::quarter_pi));
 	setRangeAngle ((float)(3.f * Constants::half_pi));
 	zoomFactor = 1.5f;
+	knobRange = kCKnobRangeDefault;
 }
 
 //------------------------------------------------------------------------
@@ -112,7 +113,7 @@ CMouseEventResult CKnobBase::onMouseDown (CPoint& where, const CButtonState& but
 
 	mouseState.modeLinear = false;
 	mouseState.entryState = value;
-	mouseState.range = kCKnobRange;
+	mouseState.range = knobRange;
 	mouseState.coef = (getMax () - getMin ()) / mouseState.range;
 	mouseState.oldButton = buttons;
 
@@ -193,7 +194,7 @@ CMouseEventResult CKnobBase::onMouseMoved (CPoint& where, const CButtonState& bu
 				CCoord diff = (mouseState.firstPoint.y - where.y) + (where.x - mouseState.firstPoint.x);
 				if (buttons != mouseState.oldButton)
 				{
-					mouseState.range = kCKnobRange;
+					mouseState.range = knobRange;
 					if (buttons & kZoomModifier)
 						mouseState.range *= zoomFactor;
 
@@ -418,7 +419,7 @@ CKnob::CKnob (const CRect& size, IControlListener* listener, int32_t tag, CBitma
 	{
 		inset = 3;
 	}
-	
+
 	colorShadowHandle = kGreyCColor;
 	colorHandle = kWhiteCColor;
 	coronaLineStyle = kLineOnOffDash;
@@ -611,7 +612,7 @@ void CKnob::drawHandleAsLine (CDrawContext* pContext) const
 	pContext->setLineStyle (CLineStyle (CLineStyle::kLineCapRound));
 	pContext->setDrawMode (kAntiAliasing | kNonIntegralMode);
 	pContext->drawLine (where, origin);
-	
+
 	where.offset (1, -1);
 	origin.offset (1, -1);
 	pContext->setFrameColor (colorHandle);
@@ -755,12 +756,11 @@ CMultiFrameBitmap for its background bitmap.
  * @param listener the listener
  * @param tag the control tag
  * @param background the background bitmap
- * @param offset unused
  */
 //------------------------------------------------------------------------
-CAnimKnob::CAnimKnob (const CRect& size, IControlListener* listener, int32_t tag, CBitmap* background, const CPoint &offset)
-: CKnobBase (size, listener, tag, background)
-, bInverseBitmap (false)
+CAnimKnob::CAnimKnob (const CRect& size, IControlListener* listener, int32_t tag,
+					  CBitmap* background)
+: CKnobBase (size, listener, tag, background), bInverseBitmap (false)
 {
 #if VSTGUI_ENABLE_DEPRECATED_METHODS
 	heightOfOneImage = size.getHeight ();
@@ -880,10 +880,9 @@ void CAnimKnob::draw (CDrawContext *pContext)
 	{
 		if (auto mfb = dynamic_cast<CMultiFrameBitmap*> (bitmap))
 		{
-			auto frameIndex = static_cast<uint16_t> (std::min (
-				mfb->getNumFrames () - 1.f, getValueNormalized () * mfb->getNumFrames ()));
+			auto frameIndex = getMultiFrameBitmapIndex (*mfb, getValueNormalized ());
 			if (bInverseBitmap)
-				frameIndex = mfb->getNumFrames () - frameIndex;
+				frameIndex = getInverseIndex (*mfb, frameIndex);
 			mfb->drawFrame (pContext, frameIndex, getViewSize ().getTopLeft ());
 		}
 		else
