@@ -12,10 +12,12 @@
 #include "../comptr.h"
 #include "../../../crect.h"
 #include "../../../cgraphicstransform.h"
+#include "../../../cbitmap.h"
 #include "../../../ccolor.h"
 #include "../../../cdrawdefs.h"
 #include "../../../clinestyle.h"
 #include <stack>
+#include <array>
 
 #include <d2d1_1.h>
 #include <dwrite.h>
@@ -37,6 +39,134 @@ struct TransformGuard
 	ID2D1DeviceContext* context;
 };
 
+//------------------------------------------------------------------------
+struct D2DBitmapDeviceContext : D2DGraphicsDeviceContext
+{
+	static PlatformGraphicsDeviceContextPtr make (const D2DGraphicsDevice& device,
+												  ID2D1DeviceContext* deviceContext,
+												  const SharedPointer<D2DBitmap>& bitmap)
+	{
+		D2D1_BITMAP_PROPERTIES1 props = D2D1::BitmapProperties1 (
+			D2D1_BITMAP_OPTIONS_TARGET | D2D1_BITMAP_OPTIONS_CANNOT_DRAW,
+			D2D1::PixelFormat (DXGI_FORMAT_B8G8R8A8_UNORM, D2D1_ALPHA_MODE_PREMULTIPLIED));
+
+		COM::Ptr<ID2D1Bitmap1> bitmapTarget;
+		deviceContext->CreateBitmapFromWicBitmap (bitmap->getSource (), &props,
+												  bitmapTarget.adoptPtr ());
+		if (!bitmapTarget)
+			return nullptr;
+
+		deviceContext->SetTarget (bitmapTarget.get ());
+
+		TransformMatrix tm;
+		tm.scale (bitmap->getScaleFactor (), bitmap->getScaleFactor ());
+		deviceContext->SetTransform (convert (tm));
+		return std::make_shared<D2DBitmapDeviceContext> (device, bitmap, deviceContext,
+														 TransformMatrix {});
+	}
+
+	D2DBitmapDeviceContext (const D2DGraphicsDevice& device, const SharedPointer<D2DBitmap>& bitmap,
+							ID2D1DeviceContext* deviceContext, const TransformMatrix& tm)
+	: D2DGraphicsDeviceContext (device, deviceContext, tm), bitmap (bitmap)
+	{
+	}
+
+	bool endDraw () const
+	{
+		if (D2DGraphicsDeviceContext::endDraw ())
+		{
+			COM::Ptr<ID2D1Image> image;
+			getID2D1DeviceContext ()->GetTarget (image.adoptPtr ());
+			if (!image)
+				return false;
+			COM::Ptr<ID2D1Bitmap1> bitmapTarget;
+			if (FAILED (image->QueryInterface<ID2D1Bitmap1> (bitmapTarget.adoptPtr ())))
+				return false;
+			auto pa = bitmap->lockPixels (true);
+			if (!pa)
+				return false;
+			auto size = bitmap->getSize ();
+			D2D1_BITMAP_PROPERTIES1 props = D2D1::BitmapProperties1 (
+				D2D1_BITMAP_OPTIONS_CPU_READ | D2D1_BITMAP_OPTIONS_CANNOT_DRAW,
+				D2D1::PixelFormat (DXGI_FORMAT_B8G8R8A8_UNORM, D2D1_ALPHA_MODE_PREMULTIPLIED));
+			COM::Ptr<ID2D1Bitmap1> cpuBitmap;
+			auto hr = getID2D1DeviceContext ()->CreateBitmap (
+				D2D1_SIZE_U {static_cast<UINT32> (size.x), static_cast<UINT32> (size.y)}, nullptr,
+				pa->getBytesPerRow (), props, cpuBitmap.adoptPtr ());
+			if (FAILED (hr))
+				return false;
+			D2D1_POINT_2U dp {};
+			D2D1_RECT_U dstRect {0, 0, static_cast<UINT32> (size.x), static_cast<UINT32> (size.y)};
+			hr = cpuBitmap->CopyFromBitmap (&dp, bitmapTarget.get (), &dstRect);
+			if (FAILED (hr))
+				return false;
+			D2D1_MAPPED_RECT mappedRect {};
+			hr = cpuBitmap->Map (D2D1_MAP_OPTIONS_READ, &mappedRect);
+			if (FAILED (hr))
+				return false;
+			auto height = static_cast<uint32_t> (size.y);
+			if (pa->getBytesPerRow () == mappedRect.pitch)
+			{
+				memcpy (pa->getAddress (), mappedRect.bits, mappedRect.pitch * height);
+			}
+			else
+			{
+				auto src = mappedRect.bits;
+				for (auto row = 0u; row < height; ++row, src += mappedRect.pitch)
+				{
+					auto dst = pa->getAddress () + pa->getBytesPerRow () * row;
+					memcpy (dst, src, pa->getBytesPerRow ());
+				}
+			}
+			cpuBitmap->Unmap ();
+			return true;
+		}
+		return false;
+	}
+
+	SharedPointer<D2DBitmap> bitmap;
+};
+
+//------------------------------------------------------------------------
+COM::Ptr<ID2D1ImageBrush> createBrushFromBitmap (ID2D1DeviceContext* deviceContext,
+												 ID2D1Bitmap* d2d1Bitmap, CRect srcRect,
+												 CRect dstRect, double alpha,
+												 BitmapInterpolationQuality quality,
+												 CGraphicsTransform* brushTM = nullptr)
+{
+	CGraphicsTransform brushTransform;
+	if (brushTM)
+		brushTransform = *brushTM;
+	brushTransform.translate (dstRect.getTopLeft ());
+
+	D2D1_IMAGE_BRUSH_PROPERTIES imageBrushProp = {};
+	imageBrushProp.sourceRectangle = convert (srcRect);
+	imageBrushProp.extendModeX = imageBrushProp.extendModeY = D2D1_EXTEND_MODE_WRAP;
+	switch (quality)
+	{
+		case BitmapInterpolationQuality::kLow:
+			imageBrushProp.interpolationMode = D2D1_INTERPOLATION_MODE_NEAREST_NEIGHBOR;
+			break;
+
+		case BitmapInterpolationQuality::kMedium:
+		case BitmapInterpolationQuality::kHigh:
+		default:
+			imageBrushProp.interpolationMode = D2D1_INTERPOLATION_MODE_LINEAR;
+			break;
+	}
+
+	D2D1_BRUSH_PROPERTIES brushProp = {};
+	brushProp.opacity = static_cast<FLOAT> (alpha);
+	brushProp.transform = convert (brushTransform);
+
+	COM::Ptr<ID2D1ImageBrush> brush;
+	auto hr =
+		deviceContext->CreateImageBrush (d2d1Bitmap, imageBrushProp, brushProp, brush.adoptPtr ());
+	if (FAILED (hr))
+		return {};
+	return brush;
+}
+
 //------------------------------------------------------------------------
 } // anonymous namespace
 
@@ -117,23 +247,7 @@ PlatformGraphicsDeviceContextPtr
 
 		D2DBitmapCache::removeBitmap (d2dBitmap);
 
-		D2D1_BITMAP_PROPERTIES1 props = D2D1::BitmapProperties1 (
-		    D2D1_BITMAP_OPTIONS_TARGET | D2D1_BITMAP_OPTIONS_CANNOT_DRAW,
-		    D2D1::PixelFormat (DXGI_FORMAT_B8G8R8A8_UNORM, D2D1_ALPHA_MODE_PREMULTIPLIED));
-
-		COM::Ptr<ID2D1Bitmap1> bitmapTarget;
-		deviceContext->CreateBitmapFromWicBitmap (d2dBitmap->getSource (), &props, bitmapTarget.adoptPtr ());
-		if (!bitmapTarget)
-			return nullptr;
-
-		deviceContext->SetTarget (bitmapTarget.get ());
-
-		TransformMatrix tm;
-		tm.scale (d2dBitmap->getScaleFactor (), d2dBitmap->getScaleFactor ());
-		deviceContext->SetTransform (convert (tm));
-
-		return std::make_shared<D2DGraphicsDeviceContext> (*this, deviceContext.get (),
-														   TransformMatrix {});
+		return D2DBitmapDeviceContext::make (*this, deviceContext.get (), d2dBitmap);
 	}
 	return nullptr;
 }
@@ -166,49 +280,19 @@ struct D2DGraphicsDeviceContext::Impl
 		auto transform = convert (tmGuard.matrix) * globalTM * state.tm;
 		transform.scale (scaleFactor, scaleFactor);
 		transform.translate (transformOffset);
-		bool useLayer = transform.m12 != 0. || transform.m21 != 0.;
-		if (useLayer)
-		{ // we have a rotated matrix, we need to use a layer
-			COM::Ptr<ID2D1Factory> factory {};
-			deviceContext->GetFactory (factory.adoptPtr ());
-			COM::Ptr<ID2D1RectangleGeometry> geometry;
-			if (SUCCEEDED (
-					factory->CreateRectangleGeometry (convert (state.clip), geometry.adoptPtr ())))
-			{
-				if (applyClip.isEmpty () == false)
-					deviceContext->PopAxisAlignedClip ();
-				deviceContext->PushLayer (D2D1::LayerParameters (D2D1::InfiniteRect (),
-																 geometry.get (),
-																 D2D1_ANTIALIAS_MODE_ALIASED),
-										  nullptr);
-				geometry->Release ();
-				applyClip = state.clip;
-			}
-			else
-			{
-				useLayer = false;
-			}
-		}
-		if (!useLayer)
+		auto newClip = state.clip;
+		globalTM.transform (newClip);
+		if (applyClip != newClip)
 		{
-			auto newClip = state.clip;
-			globalTM.transform (newClip);
-			if (applyClip != newClip)
-			{
-				if (applyClip.isEmpty () == false)
-					deviceContext->PopAxisAlignedClip ();
-				if (newClip.isEmpty () == false)
-					deviceContext->PushAxisAlignedClip (convert (newClip),
-														D2D1_ANTIALIAS_MODE_ALIASED);
-				applyClip = newClip;
-			}
+			if (applyClip.isEmpty () == false)
+				deviceContext->PopAxisAlignedClip ();
+			if (newClip.isEmpty () == false)
+				deviceContext->PushAxisAlignedClip (convert (newClip), D2D1_ANTIALIAS_MODE_ALIASED);
+			applyClip = newClip;
 		}
 		deviceContext->SetTransform (convert (transform));
 
 		p (deviceContext.get ());
-
-		if (useLayer)
-			deviceContext->PopLayer ();
 	}
 
 	//-----------------------------------------------------------------------------
@@ -462,6 +546,7 @@ bool D2DGraphicsDeviceContext::drawPolygon (const PointList& polygonPointList,
 	{
 		path->addLine (polygonPointList[i]);
 	}
+	path->finishBuilding ();
 	if (drawStyle == PlatformGraphicsDrawStyle::Filled ||
 		drawStyle == PlatformGraphicsDrawStyle::FilledAndStroked)
 	{
@@ -514,6 +599,8 @@ bool D2DGraphicsDeviceContext::drawArc (CRect rect, double startAngle1, double e
 	if (impl->state.drawMode.integralMode ())
 		pixelAlign (impl->state.tm, rect);
 	path->addArc (rect, startAngle1, endAngle2, true);
+	path->finishBuilding ();
+
 	if (drawStyle == PlatformGraphicsDrawStyle::Filled ||
 		drawStyle == PlatformGraphicsDrawStyle::FilledAndStroked)
 		drawGraphicsPath (*path, PlatformGraphicsPathDrawMode::Filled, nullptr);
@@ -598,24 +685,10 @@ bool D2DGraphicsDeviceContext::drawBitmap (IPlatformBitmap& bitmap, CRect dest,
 		source.setWidth (d2d1Bitmap->GetSize ().width);
 		source.setHeight (d2d1Bitmap->GetSize ().height);
 
-		D2D1_BITMAP_INTERPOLATION_MODE mode;
-		switch (quality)
-		{
-			case BitmapInterpolationQuality::kLow:
-				mode = D2D1_BITMAP_INTERPOLATION_MODE_NEAREST_NEIGHBOR;
-				break;
-
-			case BitmapInterpolationQuality::kMedium:
-			case BitmapInterpolationQuality::kHigh:
-			default:
-				mode = D2D1_BITMAP_INTERPOLATION_MODE_LINEAR;
-				break;
-		}
-
 		D2D1_RECT_F sourceRect = convert (source);
 		deviceContext->DrawBitmap (d2d1Bitmap, convert (d),
-								   static_cast<FLOAT> (alpha * impl->state.globalAlpha), mode,
-								   &sourceRect);
+								   static_cast<FLOAT> (alpha * impl->state.globalAlpha),
+								   convert (quality), &sourceRect);
 	});
 	setTransformMatrix (originalTransformMatrix);
 	impl->state.clip = originalClip;
@@ -854,12 +927,88 @@ const IPlatformGraphicsDeviceContextBitmapExt* D2DGraphicsDeviceContext::asBitma
 }
 
 //------------------------------------------------------------------------
-bool D2DGraphicsDeviceContext::drawBitmapNinePartTiled (IPlatformBitmap& bitmap, CRect dest,
+bool D2DGraphicsDeviceContext::drawBitmapNinePartTiled (IPlatformBitmap& bitmap, CRect dstRect,
 														const CNinePartTiledDescription& desc,
 														double alpha,
 														BitmapInterpolationQuality quality) const
 {
-	return false;
+	using NPTD = CNinePartTiledDescription;
+
+	alpha *= impl->state.globalAlpha;
+	if (alpha == 0.)
+		return true;
+
+	D2DBitmap* d2dBitmap = dynamic_cast<D2DBitmap*> (&bitmap);
+	if (!d2dBitmap || !d2dBitmap->getSource ())
+		return false;
+	auto d2d1Bitmap =
+		D2DBitmapCache::getBitmap (d2dBitmap, impl->deviceContext.get (), impl->device.get ());
+	if (!d2d1Bitmap)
+		return false;
+
+	auto interpolationMode = convert (quality);
+	auto originalClip = impl->state.clip;
+	auto cr = dstRect;
+	impl->state.tm.transform (cr);
+	cr.bound (originalClip);
+	impl->state.clip = cr;
+
+	auto originalTransformMatrix = impl->state.tm;
+
+	double bitmapScaleFactor = d2dBitmap->getScaleFactor ();
+	CGraphicsTransform bitmapTransform;
+	bitmapTransform.scale (bitmapScaleFactor, bitmapScaleFactor);
+	auto invBitmapTransform = bitmapTransform.inverse ();
+	setTransformMatrix ({});
+
+	CRect bitmapBounds (0, 0, bitmap.getSize ().x, bitmap.getSize ().y);
+	invBitmapTransform.transform (bitmapBounds);
+
+	std::array<CRect, NPTD::kPartCount> mySourceRect;
+	std::array<CRect, NPTD::kPartCount> myDestRect;
+
+	desc.calcRects (bitmapBounds, mySourceRect.data ());
+	desc.calcRects (dstRect, myDestRect.data ());
+
+	std::for_each (mySourceRect.begin (), mySourceRect.end (),
+				   [&] (auto& el) { bitmapTransform.transform (el); });
+	std::for_each (myDestRect.begin (), myDestRect.end (),
+				   [&] (auto& el) { originalTransformMatrix.transform (el); });
+
+	impl->doInContext ([&] (auto deviceContext) {
+		auto drawBitmap = [&] (const CRect& source, const CRect& destination) {
+			auto sourceRect = convert (source);
+			deviceContext->DrawBitmap (d2d1Bitmap, convert (destination),
+									   static_cast<FLOAT> (alpha), interpolationMode, &sourceRect);
+		};
+
+		drawBitmap (mySourceRect[NPTD::kPartTopLeft], myDestRect[NPTD::kPartTopLeft]);
+		drawBitmap (mySourceRect[NPTD::kPartTopRight], myDestRect[NPTD::kPartTopRight]);
+		drawBitmap (mySourceRect[NPTD::kPartBottomLeft], myDestRect[NPTD::kPartBottomLeft]);
+		drawBitmap (mySourceRect[NPTD::kPartBottomRight], myDestRect[NPTD::kPartBottomRight]);
+
+		auto tm = originalTransformMatrix * invBitmapTransform;
+		tm.dx = tm.dy = 0;
+
+		auto drawBitmapBrush = [&] (CRect src, CRect dst) {
+			if (auto brush = createBrushFromBitmap (deviceContext, d2d1Bitmap, src, dst, alpha,
+													quality, &tm))
+			{
+				dst.makeIntegral ();
+				deviceContext->FillRectangle (convert (dst), brush.get ());
+			}
+		};
+
+		drawBitmapBrush (mySourceRect[NPTD::kPartTop], myDestRect[NPTD::kPartTop]);
+		drawBitmapBrush (mySourceRect[NPTD::kPartCenter], myDestRect[NPTD::kPartCenter]);
+		drawBitmapBrush (mySourceRect[NPTD::kPartLeft], myDestRect[NPTD::kPartLeft]);
+		drawBitmapBrush (mySourceRect[NPTD::kPartRight], myDestRect[NPTD::kPartRight]);
+		drawBitmapBrush (mySourceRect[NPTD::kPartBottom], myDestRect[NPTD::kPartBottom]);
+	});
+
+	setTransformMatrix (originalTransformMatrix);
+	impl->state.clip = originalClip;
+	return true;
 }
 
 //------------------------------------------------------------------------
@@ -887,32 +1036,9 @@ bool D2DGraphicsDeviceContext::fillRectWithBitmap (IPlatformBitmap& bitmap, CRec
 	invBitmapTransform.transform (dstRect);
 	invBitmapTransform.transform (srcRect);
 
-	D2D1_IMAGE_BRUSH_PROPERTIES imageBrushProp = {};
-	imageBrushProp.sourceRectangle = convert (srcRect);
-	imageBrushProp.extendModeX = imageBrushProp.extendModeY = D2D1_EXTEND_MODE_WRAP;
-	switch (quality)
-	{
-		case BitmapInterpolationQuality::kLow:
-			imageBrushProp.interpolationMode = D2D1_INTERPOLATION_MODE_NEAREST_NEIGHBOR;
-			break;
-
-		case BitmapInterpolationQuality::kMedium:
-		case BitmapInterpolationQuality::kHigh:
-		default:
-			imageBrushProp.interpolationMode = D2D1_INTERPOLATION_MODE_LINEAR;
-			break;
-	}
-	CGraphicsTransform brushTransform;
-	brushTransform.translate (dstRect.getTopLeft ());
-
-	D2D1_BRUSH_PROPERTIES brushProp = {};
-	brushProp.opacity = 1.f;
-	brushProp.transform = convert (brushTransform);
-
-	COM::Ptr<ID2D1ImageBrush> brush;
-	auto hr = impl->deviceContext->CreateImageBrush (d2d1Bitmap, imageBrushProp, brushProp,
-													 brush.adoptPtr ());
-	if (FAILED (hr))
+	auto brush = createBrushFromBitmap (impl->deviceContext.get (), d2d1Bitmap, srcRect, dstRect,
+										alpha * impl->state.globalAlpha, quality);
+	if (!brush)
 		return false;
 
 	impl->state.clip = cr;
